@@ -7,11 +7,11 @@ use Devel::StackTrace;
 use IO::File;
 use UNIVERSAL::require;
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 
-our $A_HEAD = '<a class="package" href="?rm=view_code&amp;module=';
-our $A_HEAD_POD = '<a class="package" href="?rm=view_pod&amp;module=';
-our $A_TAIL = ';view_code_no_popup=1">';
+our $A_CODE = '<a class="package" href="?rm=view_code&amp;module=';
+our $A_POD  = '<a class="package" href="?rm=view_pod&amp;module=';
+our $A_TAIL = '">';
 our $A_END  = '</a>';
 
 our $TEMPLATE = qq{
@@ -64,6 +64,7 @@ our $TEMPLATE = qq{
            }
            div.url {
                font-size: x-small;
+               overflow: auto;
            }
            pre {
                font-size: .8em;
@@ -133,7 +134,7 @@ sub import {
 
     $caller->add_callback( 'init', sub{
         my $self = shift;
-        my $de;
+
         $SIG{__DIE__} = sub{
             push @{$self->{__stacktrace}},[Devel::StackTrace->new(ignore_package=>[qw/CGI::Application::Plugin::DebugScreen Carp CGI::Carp/])->frames];
             die @_; # rethrow
@@ -141,26 +142,30 @@ sub import {
         {
             no strict 'refs';
             *{"$caller\::report"} = \&debug_report;
+
+            *{"$caller\::__debugscreen_error"} = sub{
+                my $self = shift;
+                if (
+                      exists $INC{'CGI/Application/Plugin/ViewCode.pm'}
+                      &&
+                    ! exists $INC{'CGI/Application/Dispatch.pm'}
+                   )
+                {
+                    $self->{__viewcode}++;
+                }
+                $self->report(@_);
+            };
         }
     });
 
     $caller->add_callback( 'error', sub{
         my $self = shift;
-        if (
-              exists $INC{'CGI/Application/Plugin/ViewCode.pm'}
-              &&
-            ! exists $INC{'CGI/Application/Dispatch.pm'}
-           )
-        {
-            $self->{__viewcode}++;
+        if ( $ENV{CGI_APP_DEBUG} && exists $INC{'CGI/Application/Plugin/ViewCode.pm'} ) {
+            $self->error_mode('__debugscreen_error');
         }
-        $self->report(@_);
     });
 
-    if ( ! exists $INC{'CGI/Application/Dispatch.pm'} &&
-         ! exists $INC{'CGI/Application/Plugin/ViewCode.pm'}
-       )
-    {
+    if ( ! exists $INC{'CGI/Application/Plugin/ViewCode.pm'} ) {
         "CGI::Application::Plugin::ViewCode"->require
          or delete $INC{'CGI/Application/Plugin/ViewCode.pm'};
         unless ($@) {goto &CGI::Application::Plugin::ViewCode::import}
@@ -170,14 +175,15 @@ sub import {
 sub debug_report{
     my $self = shift;
     my $desc = shift;
-    my $url = $self->query->url;
+    my $url = $self->query->url(-path_info=>1,-query=>1);
+
     my $title = ref $self || $self;
 
     $title = html_escape($title);
     my $title_a = $title;
 
     if ( $self->{__viewcode} ) {
-        $title_a = $A_HEAD . $title . $A_TAIL . $title . $A_END;
+        $title_a = $A_CODE . $title . $A_TAIL . $title . $A_END;
     }
 
     my $stacks = $self->{__stacktrace}[0];
@@ -191,13 +197,15 @@ sub debug_report{
         $s{package}  = html_escape($s{package});
         $s{filename} = html_escape($s{filename});
         $s{line}     = html_escape($stack->{line});
-        $s{code_preview} = print_context($s{filename},$s{line});
+        $s{code_preview} = print_context($s{filename},$s{line},$s{package},$self->{__viewcode});
 
         if ( $self->{__viewcode} && $s{package} ne 'main' ) {
-            $s{code} = $A_HEAD . $s{package} . $A_TAIL . "code" . $A_END;
-            $s{pod}  = $A_HEAD_POD . $s{package} . $A_TAIL . "pod" . $A_END;
-            $s{filename} = $A_HEAD . $s{package} . $A_TAIL . $s{filename} . $A_END;
-            $s{package} = $A_HEAD . $s{package} . $A_TAIL . $s{package} . $A_END;
+            $s{line}     = $A_CODE . $s{package} . '#'.$s{line} . $A_TAIL . $s{line} . $A_END;
+            $s{pod}      = $A_POD  . $s{package} . $A_TAIL . 'pod'        . $A_END;
+            $s{code}     = $A_CODE . $s{package} . $A_TAIL . 'code'       . $A_END;
+            $s{filename} = $A_CODE . $s{package} . $A_TAIL . $s{filename} . $A_END;
+            $s{package}  = $A_CODE . $s{package} . $A_TAIL . $s{package}  . $A_END;
+
             $s{view} = $self->{__viewcode};
         }
         push @stacktraces, \%s;
@@ -217,12 +225,11 @@ sub debug_report{
     );
 
     $self->header_props( -type => 'text/html' );
-    my $headers = $self->_send_headers();
-    print $headers.$t->output;
+    return $t->output;
 }
 
 sub print_context {
-    my($file, $linenum) = @_;
+    my($file, $linenum, $package, $view) = @_;
     my $code;
     if (-f $file) {
         my $start = $linenum - 3;
@@ -235,10 +242,19 @@ sub print_context {
                 last if $cur_line > $end;
                 next if $cur_line < $start;
                 my @tag = $cur_line == $linenum ? qw(<b> </b>) : ("","");
-                $code .= sprintf(
-                    '%s%5d: %s%s',
-                        $tag[0], $cur_line, html_escape($line), $tag[1],
-                );
+                if ( $view && $package ne 'main' && $cur_line == $linenum ) {
+                    my $t_line = $A_CODE.$package.'#'.$linenum.$A_TAIL;
+                    $code .= sprintf(
+                        '%s%s%5d: %s%s%s',
+                            $tag[0], $t_line, $cur_line, html_escape($line), $A_END, $tag[1],
+                    );
+                }
+                else {
+                    $code .= sprintf(
+                        '%s%5d: %s%s',
+                            $tag[0], $cur_line, html_escape($line), $tag[1],
+                    );
+                }
             }
         }
     }
@@ -262,7 +278,7 @@ CGI::Application::Plugin::DebugScreen - add Debug support to CGI::Application.
 
 =head1 VERSION
 
-This documentation refers to CGI::Application::Plugin::DebugScreen version 0.05
+This documentation refers to CGI::Application::Plugin::DebugScreen version 0.06
 
 =head1 SYNOPSIS
 
@@ -276,18 +292,25 @@ If "Internal Server Error" was generated by "run_mode"....
 This plug-in add Debug support to CGI::Application.
 This plug-in like Catalyst debug mode.
 
+DebugScreen is done when B<$ENV{CGI_APP_DEBUG}> is set,
+ and DebugScreen is not done when not setting it.
+When your code is released, this plug-in need not be removed.
+
 When 'die' is generated by 'run_mode',
  this plug-in outputs the stack trace by error_mode().
+As for this plug-in, error_mode() is overwrited in error callback.
 The error cannot be caught excluding run_mode.
 
 This uses CGI::Application::Plugin::ViewCode
- if a state that CGI::Application::Plugin::ViewCode can be used or used. 
+ if a state that CGI::Application::Plugin::ViewCode can be used or used.
 But CGI::Application::Dispatch is used,
  this not uses CGI::Application::Plugin::ViewCode.
 
 When CGI::Application::Plugin::ViewCode can be used,
- Title, Package, File, and code are links to CGI::Application::Plugin::ViewCode's view_code mode.
+ Title, Package, File, code and line are links to CGI::Application::Plugin::ViewCode's view_code mode.
+line jumps to the specified line.
 And pod are links to CGI::Application::Plugin::ViewCode's view_pod mode.
+The code of the displayed is links to CGI::Application::Plugin::ViewCode's view_code mode.
 
 =head1 DEPENDENCIES
 
@@ -315,21 +338,13 @@ Patches are welcome.
 
 =head1 SEE ALSO
 
-L<strict>
-
-L<warnings>
-
-L<CGI::Application>
-
-L<HTML::Template>
-
-L<Devel::StackTrace>
-
-L<IO::File>
-
 L<CGI::Application::Plugin::ViewCode>
 
-L<UNIVERSAL::require>
+L<Sledge::Plugin::DebugScreen>
+
+L<CGI::Carp::DebugScreen>
+
+L<Catalyst::Plugin::StackTrace>
 
 =head1 Thanks To
 
@@ -347,7 +362,7 @@ Atsushi Kobayashi, E<lt>nekokak@cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2005 by Atsushi Kobayashi (E<lt>nekokak@cpan.orgE<gt>). All rights reserved.
+Copyright (C) 2006 by Atsushi Kobayashi (E<lt>nekokak@cpan.orgE<gt>). All rights reserved.
 
 This library is free software; you can redistribute it and/or modify it
  under the same terms as Perl itself. See L<perlartistic>.
